from django.apps import AppConfig
from django.utils.translation import gettext_lazy


class PluginApp(AppConfig):
    name = "pretalx_hitalx"
    verbose_name = "Various extensions to make the Life easier for the HI Congress"

    class PretalxPluginMeta:
        name = gettext_lazy("Pretalx Hitalx")
        author = "Jonatan Zint"
        description = gettext_lazy(
            "Various features specifically for the Hedonist International"
        )
        visible = True
        version = "0.0.9"

    def ready(self):
        from . import signals  # NOQA



from functools import cached_property

from django.contrib.contenttypes.models import ContentType
from django.db.models import Count, Exists, OuterRef, Q
from django.http import HttpResponseRedirect
from django.shortcuts import reverse, get_object_or_404
from django.views.generic import ListView, View, DeleteView
from django_context_decorator import context
from pretalx.common.mixins.views import (
    ActionFromUrl,
    EventPermissionRequired,
    Filterable,
    PermissionRequired,
    Sortable,
)
from pretalx.common.models import ActivityLog
from pretalx.common.views import CreateOrUpdateView
from pretalx.event.models import Event
from pretalx.person.forms import SpeakerFilterForm
from pretalx.person.models import SpeakerProfile
from pretalx.submission.models import Answer, SubmissionStates

from .form import SpeakerExpenseForm, SpeakerToursForm, TourForm
from .models import ExpenseItem, Tour


class SpeakerList(EventPermissionRequired, Sortable, Filterable, ListView):
    model = SpeakerProfile
    template_name = "pretalx_hitalx/speakers.html"
    context_object_name = "speakers"
    default_filters = ("user__email__icontains", "user__name__icontains")
    sortable_fields = ("user__email", "user__name")
    default_sort_field = "user__name"
    paginate_by = 25
    permission_required = "orga.view_speakers"

    @context
    def filter_form(self):
        return SpeakerFilterForm(self.request.event, self.request.GET)

    def get_queryset(self):
        qs = (
            SpeakerProfile.objects.filter(
                event=self.request.event, user__in=self.request.event.submitters
            )
            .select_related("event", "user")
            .annotate(
                submission_count=Count(
                    "user__submissions",
                    filter=Q(user__submissions__event=self.request.event),
                ),
                accepted_submission_count=Count(
                    "user__submissions",
                    filter=Q(user__submissions__event=self.request.event)
                    & Q(user__submissions__state__in=["accepted", "confirmed"]),
                ),
            )
        )

        qs = self.filter_queryset(qs)
        if "role" in self.request.GET:
            if self.request.GET["role"] == "true":
                qs = qs.filter(
                    user__submissions__in=self.request.event.submissions.filter(
                        state__in=[
                            SubmissionStates.ACCEPTED,
                            SubmissionStates.CONFIRMED,
                        ]
                    )
                )
            elif self.request.GET["role"] == "false":
                qs = qs.exclude(
                    user__submissions__in=self.request.event.submissions.filter(
                        state__in=[
                            SubmissionStates.ACCEPTED,
                            SubmissionStates.CONFIRMED,
                        ]
                    )
                )

        question = self.request.GET.get("question")
        unanswered = self.request.GET.get("unanswered")
        answer = self.request.GET.get("answer")
        option = self.request.GET.get("answer__options")
        if question and (answer or option):
            if option:
                answers = Answer.objects.filter(
                    person_id=OuterRef("user_id"),
                    question_id=question,
                    options__pk=option,
                )
            else:
                answers = Answer.objects.filter(
                    person_id=OuterRef("user_id"),
                    question_id=question,
                    answer__exact=answer,
                )
            qs = qs.annotate(has_answer=Exists(answers)).filter(has_answer=True)
        elif question and unanswered:
            answers = Answer.objects.filter(
                question_id=question, person_id=OuterRef("user_id")
            )
            qs = qs.annotate(has_answer=Exists(answers)).filter(has_answer=False)
        qs = qs.order_by("id").distinct()
        qs = self.sort_queryset(qs)
        return qs


class SpeakerExpenseList(EventPermissionRequired, ListView):
    model = ExpenseItem
    template_name = "pretalx_hitalx/speaker_expenses.html"
    context_object_name = "speakers"
    default_sort_field = "user__name"
    paginate_by = 25
    permission_required = "orga.view_speakers"

    def get_queryset(self):
        return super().get_queryset().filter(speaker_id=self.kwargs["speaker_id"])

    @context
    def speaker_profile(self):
        return SpeakerProfile.objects.filter(
            user=self.kwargs["speaker_id"], event=self.request.event
        ).first()


class SpeakerExpenseDetail(PermissionRequired, ActionFromUrl, CreateOrUpdateView):
    model = ExpenseItem
    form_class = SpeakerExpenseForm
    template_name = "pretalx_hitalx/speaker_expense.html"
    permission_required = "orga.view_speaker"
    write_permission_required = "orga.change_speaker"

    def get_form(self, form_class=None):
        if form_class is None:
            form_class = self.get_form_class()
        return form_class(
            **{
                **self.get_form_kwargs(),
                "initial": {"speaker": self.kwargs["speaker_id"]},
            }
        )

    def post(self, request, *args, **kwargs):
        created = False if self.object else True
        response = super(SpeakerExpenseDetail, self).post(request, *args, **kwargs)
        self.object.event = request.event
        if created:
            self.object.log_action(
                "pretalx_hitalx.expense_item.create",
                person=request.user,
                data={"amount": self.object.amount, "notes": self.object.notes},
            )
        else:
            self.object.log_action(
                "pretalx_hitalx.expense_item.edit",
                person=request.user,
                data={"amount": self.object.amount, "notes": self.object.notes},
            )
        return response

    @property
    def permission_object(self):
        return SpeakerProfile.objects.filter(
            user=self.kwargs["speaker_id"], event=self.request.event
        ).first()

    def get_permission_object(self):
        return self.permission_object

    def get_object(self):
        return ExpenseItem.objects.filter(pk=self.kwargs.get("pk")).first()

    @cached_property
    def object(self):
        return self.get_object()

    def get_success_url(self) -> str:
        return reverse(
            "plugins:pretalx_hitalx:expenses.view",
            kwargs={
                "event": self.request.event.slug,
                "speaker_id": self.kwargs["speaker_id"],
            },
        )

    @context
    def log_entries(self, **kwargs):
        if self.object:
            return ActivityLog.objects.filter(
                event=self.request.event,
                content_type=ContentType.objects.get_for_model(ExpenseItem),
                object_id=self.object.id,
            )
        else:
            return []


class MarkExpenseView(PermissionRequired, View):
    model = ExpenseItem
    context_object_name = "expense_item"
    permission_required = "orga.view_speaker"
    write_permission_required = "orga.change_speaker"

    def get_object(self):
        return self.object

    @cached_property
    def object(self):
        return ExpenseItem.objects.filter(pk=self.kwargs.get("pk")).first()

    @property
    def permission_object(self):
        return SpeakerProfile.objects.filter(
            user=self.kwargs["speaker_id"], event=self.request.event
        ).first()

    def get_permission_object(self):
        return self.permission_object

    def post(self, request, *args, **kwargs):
        obj = self.get_object()
        paid = request.POST.get("paid", False) == "true"
        obj.paid = paid
        obj.save()
        self.object.event = request.event
        obj.log_action(
            "pretalx_hitalx.expense_item.mark",
            person=request.user,
            data={"paid": self.object.paid},
        )
        referer = request.META.get("HTTP_REFERER", None)
        if referer:
            return HttpResponseRedirect(referer)
        else:
            return HttpResponseRedirect(
                reverse(
                    "plugins:pretalx_hitalx:expenses.view",
                    kwargs={
                        "event": self.request.event.slug,
                        "speaker_id": self.kwargs["speaker_id"],
                    },
                )
            )


class SpeakerTourManagement(PermissionRequired, ActionFromUrl, CreateOrUpdateView):
    template_name = "pretalx_hitalx/speaker_tours.html"
    form_class = SpeakerToursForm
    model = SpeakerProfile
    permission_required = "orga.view_speaker"
    write_permission_required = "orga.change_speaker"

    def get_success_url(self) -> str:
        return reverse(
            "plugins:pretalx_hitalx:speakers_by_expense.view",
            kwargs={
                "event": self.request.event.slug
            },
        )


class TourListView(EventPermissionRequired, Sortable, Filterable, ListView):
    template_name = "pretalx_hitalx/tours.html"
    model = Tour
    context_object_name = "tours"
    filter_fields = ['type']
    permission_required = "orga.view_speaker"

    def get_queryset(self):
        qs = Tour.objects.filter(event__slug=self.request.event.slug)
        qs = self.filter_queryset(qs)
        return qs


class TourDetailView(EventPermissionRequired, Sortable, CreateOrUpdateView):
    template_name = "pretalx_hitalx/tour.html"
    model = Tour
    form_class = TourForm
    permission_required = "orga.view_speaker"

    def get_form(self, form_class=None):
        if form_class is None:
            form_class = self.get_form_class()
        return form_class(
            **{
                **self.get_form_kwargs(),
                "initial": {"event": Event.objects.filter(slug=self.kwargs["event"]).first().id},
            }
        )

    def get_object(self):
        return Tour.objects.filter(pk=self.kwargs.get("pk")).first()

    @cached_property
    def object(self):
        return self.get_object()

    def get_success_url(self) -> str:
        return reverse(
            "plugins:pretalx_hitalx:tours.view",
            kwargs={
                "event": self.request.event.slug
            },
        )


class TourDeleteView(EventPermissionRequired, DeleteView):
    permission_required = "orga.view_speaker"

    def get_object(self):
        return get_object_or_404(
            Tour.objects.all(), event=self.request.event, pk=self.kwargs.get("pk")
        )

    def get_success_url(self) -> str:
        return reverse(
            "plugins:pretalx_hitalx:tours.view",
            kwargs={
                "event": self.request.event.slug
            },
        )



from django.urls import path

from .views import (
    MarkExpenseView,
    SpeakerExpenseDetail,
    SpeakerExpenseList,
    SpeakerList,
    SpeakerTourManagement,
    TourListView,
    TourDetailView,
    TourDeleteView
)

urlpatterns = [
    path(
        "orga/event/<slug:event>/speakers-by-expense/",
        view=SpeakerList.as_view(),
        name="speakers_by_expense.view",
    ),
    path(
        "orga/event/<slug:event>/speakers/<int:pk>/tours/",
        view=SpeakerTourManagement.as_view(),
        name="speaker_tours.view",
    ),
    path(
        "orga/event/<slug:event>/tours/",
        view=TourListView.as_view(),
        name="tours.view",
    ),
    path(
        "orga/event/<slug:event>/tours/<int:pk>",
        view=TourDetailView.as_view(),
        name="tour.view",
    ),
    path(
        "orga/event/<slug:event>/tours/<int:pk>/delete",
        view=TourDeleteView.as_view(),
        name="tour.delete",
    ),
    path(
        "orga/event/<slug:event>/tours/new",
        view=TourDetailView.as_view(),
        name="tour.create",
    ),
    path(
        "orga/event/<slug:event>/speakers/<int:speaker_id>/expenses/",
        view=SpeakerExpenseList.as_view(),
        name="expenses.view",
    ),
    path(
        "orga/event/<slug:event>/speakers/<int:speaker_id>/expenses/new",
        view=SpeakerExpenseDetail.as_view(),
        name="expenses.create",
    ),
    path(
        "orga/event/<slug:event>/speakers/<int:speaker_id>/expense/<int:pk>",
        view=SpeakerExpenseDetail.as_view(),
        name="expense.view",
    ),
    path(
        "orga/event/<slug:event>/speakers/<int:speaker_id>/expense/<int:pk>/mark",
        view=MarkExpenseView.as_view(),
        name="expense.mark",
    ),
]



from django import forms
from django.forms import CharField, ModelForm
from django_scopes.forms import SafeModelMultipleChoiceField
from django_scopes import scope
from pretalx.common.mixins.forms import ReadOnlyFlag
from pretalx.event.models import Event
from pretalx.person.models import SpeakerProfile

from .models import ExpenseItem
from .models import Tour


class SpeakerExpenseForm(ReadOnlyFlag, ModelForm):
    description = CharField()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["speaker"].disabled = True
        self.fields["paid"].disabled = True

    class Meta:
        model = ExpenseItem
        fields = ["speaker", "description", "amount", "paid", "reference", "notes"]
        readonly_fields = ["speaker", "paid"]


class SpeakerToursForm(ReadOnlyFlag, ModelForm):
    tours = forms.ModelMultipleChoiceField(
        queryset=Tour.objects.all().order_by('departure_time'),
        widget=forms.SelectMultiple(attrs={"class": "select2"}),
    )

    def save(self, *args, **kwargs):
        instance = self.instance
        if "tours" in self.changed_data:
            instance.tours.set(self.cleaned_data["tours"])

    def __init__(self, *args, **kwargs):
        super(SpeakerToursForm, self).__init__(*args, **kwargs)
        self.initial['tours'] = kwargs['instance'].tours.all()

    class Meta:
        model = SpeakerProfile
        fields = ["tours"]
        field_classes = {
            "tours": SafeModelMultipleChoiceField,
        }


class PassengerChoiceField(forms.ModelMultipleChoiceField):
    def label_from_instance(self, obj):
        return obj.user.get_display_name()


class TourForm(ReadOnlyFlag, ModelForm):
    description = CharField()
    start_location = CharField()
    departure_time = forms.DateTimeField()
    passengers = PassengerChoiceField(
        queryset=SpeakerProfile.objects.none(),
        widget=forms.SelectMultiple(attrs={"class": "select2"}),
        blank=True,
        required=False
    )

    def __init__(self, *args, user=None, locales=None, organiser=None, **kwargs):
        super().__init__(*args, **kwargs)
        with scope(event=Event.objects.get(id=kwargs['initial'].get('event', None))):
            self.fields["passengers"].queryset = SpeakerProfile.objects.all()
        self.fields["departure_time"].widget.attrs["class"] = "datetimepickerfield"

    class Meta:
        model = Tour
        fields = ["description", "departure_time", "start_location", "type", "event", "passengers"]
        widgets = {'event': forms.HiddenInput(), 'passengers': forms.SelectMultiple()}
        field_classes = {
            "passengers": SafeModelMultipleChoiceField,
        }



import os
from distutils.command.build import build

from django.core import management
from setuptools import find_packages, setup

try:
    with open(
        os.path.join(os.path.dirname(__file__), "README.rst"), encoding="utf-8"
    ) as f:
        long_description = f.read()
except FileNotFoundError:
    long_description = ""


class CustomBuild(build):
    def run(self):
        management.call_command("compilemessages", verbosity=1)
        build.run(self)


cmdclass = {"build": CustomBuild}


setup(
    name="pretalx-hitalx",
    version="0.0.9",
    description="Some utilities specifically for the Hedonist International",
    long_description=long_description,
    url="https://github.com/l0rn/pretalx-hitlax",
    author="Jonatan Zint",
    author_email="j.zint@frykk.de",
    license="Apache Software License",
    install_requires=[
        'django-autocomplete-light'
    ],
    packages=find_packages(exclude=["tests", "tests.*"]),
    include_package_data=True,
    cmdclass=cmdclass,
    entry_points="""
[pretalx.plugin]
pretalx_hitalx=pretalx_hitalx:PretalxPluginMeta
""",
)



# Register your receivers here
from django.dispatch import receiver
from django.urls import resolve, reverse
from django.utils.translation import ugettext as _
from pretalx.common.signals import activitylog_display
from pretalx.orga.signals import nav_event


@receiver(nav_event, dispatch_uid="hitalx_nav")
def navbar_info(sender, request, **kwargs):
    url = resolve(request.path_info)
    return [
        {
            "label": _("Speaker expenses"),
            "icon": "dollar",
            "url": reverse(
                "plugins:pretalx_hitalx:speakers_by_expense.view",
                kwargs={
                    "event": request.event.slug,
                },
            ),
            "active": url.namespace == "plugins:pretalx_hitalx" and url.url_name == "speakers_by_expense.view",
        },
        {
            'label': _('Tours'),
            'icon': 'bus',
            'url': reverse('plugins:pretalx_hitalx:tours.view', kwargs={
                'event': request.event.slug,
            }),
            'active': url.namespace == 'plugins:pretalx_hitalx' and url.url_name == 'tours.view',
        }
    ]


PLUGIN_ACTION_TYPES = {
    "pretalx_hitalx.expense_item.edit": _("Speaker expense was edited"),
    "pretalx_hitalx.expense_item.create": _("Speaker expense was created"),
    "pretalx_hitalx.expense_item.mark": _("Expense paid status was changed"),
}


@receiver(activitylog_display)
def default_activitylog_display(sender, activitylog, **kwargs):
    if activitylog.action_type in PLUGIN_ACTION_TYPES:
        return f"{PLUGIN_ACTION_TYPES[activitylog.action_type]} ({activitylog.data})"

